// ApogeeNet.cpp : Library of basic networking functions for Apogee APn/Alta.
//

#include "ApogeeNet.h"
#include "ApogeeNetErr.h"

// #include "udprcv.h"



// Global variables used in this DLL
HINTERNET	g_hSession;
ULONG		g_NetImgSizeBytes;
BOOLEAN		g_NetImgInProgress;
BOOLEAN		g_FastDownload;
char		g_HostAddr[80];


// This is an example of an exported function.
APN_NET_TYPE ApnNetConnect( char *HostName )
{
	DWORD		dwBytesAvailable, dwBytesRead;
	HINTERNET	hService;
	int			RetLength;
	char		szUrl[80];
	char		*szStopStr;
	char		*lpBuffer;

	
	strcpy( g_HostAddr, HostName );

	g_hSession = InternetOpen( "ApogeeNet", 
				  			   INTERNET_OPEN_TYPE_DIRECT, 
							   NULL, 
							   NULL, 
							   0 );

	if ( g_hSession == NULL )
	{
		return APNET_ERR_CONNECT;
	}

	sprintf( szUrl, "%s%s%s", HTTP_PREAMBLE, g_HostAddr, SESSION_OPEN );

	hService = InternetOpenUrl( g_hSession, szUrl, NULL, 0, 0, 0 );

	if ( hService == NULL )
	{
		return APNET_ERR_GENERIC_CGI;		// Failure
	}
	else
	{
		InternetQueryDataAvailable(hService, &dwBytesAvailable, 0, 0);
		lpBuffer = (char *)malloc(dwBytesAvailable+1);
		InternetReadFile(hService, lpBuffer, dwBytesAvailable, &dwBytesRead);
		lpBuffer[dwBytesRead] = NULL;

		RetLength = strlen( SESSION_OPEN_RETVAL );

		if ( strncmp( lpBuffer, SESSION_OPEN_RETVAL, RetLength ) != 0 )
		{
			free( lpBuffer );
			return APNET_ERR_CONNECT;
		}

		free( lpBuffer );
		InternetCloseHandle( hService );
	}

	g_NetImgInProgress	= FALSE;
	g_NetImgSizeBytes	= 0;

	return APNET_SUCCESS;
}


APN_NET_TYPE ApnNetClose( void )
{
	DWORD		dwBytesAvailable, dwBytesRead;
	HINTERNET	hService;
	char		szUrl[80];
	char		*szStopStr;
	char		*lpBuffer;

	
	// First make sure we have a valid session already
	if ( g_hSession == NULL)
	{
		// Return successful since there is no session open anyway
		return APNET_SUCCESS;
	}

	sprintf( szUrl, "%s%s%s", HTTP_PREAMBLE, g_HostAddr, SESSION_CLOSE );

	hService = InternetOpenUrl( g_hSession, szUrl, NULL, 0, 0, 0 );

	if ( hService == NULL )
	{
		return APNET_ERR_GENERIC_CGI;		// Failure
	}
	else
	{
		InternetQueryDataAvailable(hService, &dwBytesAvailable, 0, 0);
		lpBuffer = (char *)malloc(dwBytesAvailable+1);
		InternetReadFile(hService, lpBuffer, dwBytesAvailable, &dwBytesRead);
		lpBuffer[dwBytesRead] = NULL;

		if ( strcmp( lpBuffer, SESSION_CLOSE_RETVAL ) != 0 )
		{
			free( lpBuffer );
			return APNET_ERR_CLOSE;
		}

		free( lpBuffer );
		InternetCloseHandle( hService );
	}

	InternetCloseHandle( g_hSession ); 

	if ( g_NetImgInProgress )
	{
		// Add cleanup code
		g_NetImgInProgress = FALSE;
	}

	g_hSession = NULL;

	return APNET_SUCCESS;
}


/* ----------------------------------------------------------------------------
inet_aton
---------------------------------------------------------------------------- */
/*
int inet_aton(const char *strptr, struct in_addr *addrptr) {
  unsigned long a;
  
  
  a = inet_addr(strptr);
  if (a == INADDR_NONE) return (0);
  addrptr->s_addr = a;
  return (1);
} /* inet_aton */

// #endif /* #if (defined(_WIN32)) */


/* ----------------------------------------------------------------------------
inet_pton
---------------------------------------------------------------------------- */
/*
int inet_pton(int family, const char *strptr, void *addrptr) {
  if (family == AF_INET) {
    struct in_addr in_val;

    if (inet_aton(strptr, &in_val)) {
      memcpy(addrptr, &in_val, sizeof(struct in_addr));
      return (1);
    }
    return (0);
  }
#if defined(_WIN32)
  /* TBD: _WIN32 support */
/*
#else
  errno = EAFNOSUPPORT;
#endif
  return (-1);
} /* inet_pton */

APN_NET_TYPE ApnNetDiscovery( unsigned long		Subnet,
						  unsigned short	*CameraCount,
						  char				*CameraList )
{
	WSADATA				WinSockData;
	int					LineLen, UdpPacketLen;
	int					sd;
	struct sockaddr_in  servaddr;
	struct in_addr		in_val;
	const int			on = 1;
	char				Line[DISCOVERY_MAXLINESIZE + 1];
	char				pUdpPacket[DISCOVERY_MAXBUFFER + 1];
	char				szTemp[256];
	char*				CurCamList;
	clock_t				ct;
	unsigned long		addr;

	
	// Initial count of cameras on network should be zero
	*CameraCount = 0;

	// Try to start the Windows sockets library
	if ( WSAStartup(MAKEWORD(1, 0), &WinSockData) != 0 )
	{
		return 1;
	}

	memset(&servaddr, 0, sizeof(servaddr));

	servaddr.sin_family			= AF_INET;
	servaddr.sin_port			= htons( APOGEE_IP_PORT_NUMBER );
	servaddr.sin_addr.s_addr	= Subnet;

	sd = socket( AF_INET, SOCK_DGRAM, 0 );
	
	setsockopt( sd, SOL_SOCKET, SO_BROADCAST, (char*) &on, sizeof(on) );

	LineLen = _snprintf( Line, 
						 DISCOVERY_MAXLINESIZE, 
						 "Discovery::Request-Except: \"%s\"; 0x%lX; %ld; %ld; %ld; %ld" APOGEE_CRLF APOGEE_CRLF,
						 "Apogee", 
						 0x12345678, 
						 0, 
						 DISCOVERY_TIMEOUT_SECS / 2, 
						 0, 0 );

	// Build the header for the message
	UdpPacketLen = _snprintf( pUdpPacket, 
							  DISCOVERY_MAXBUFFER, 
							  "MIME-Version: 1.0" APOGEE_CRLF
							  "Content-Type: application/octet-stream" APOGEE_CRLF
							  "Content-Transfer-Encoding: binary" APOGEE_CRLF
							  "Content-Length: 0x%X" APOGEE_CRLF
							  "X-Project: Apogee" APOGEE_CRLF
							  "X-Project-Version: 0.1" APOGEE_CRLF APOGEE_CRLF,
							  LineLen );
													

	memcpy(pUdpPacket + UdpPacketLen, Line, LineLen);
	UdpPacketLen += LineLen;

	sendto( sd, pUdpPacket, UdpPacketLen, 0, (struct sockaddr *)&servaddr, sizeof(servaddr) );

	// Wait for UDP replies
	struct	timeval tv;
	fd_set	rset;
	int		n;
	char	szResponseToken[DISCOVERY_TOKEN_ID_LENGTH];
	
	char	szId[DISCOVERY_TOKEN_LENGTH];
	char	szTag[DISCOVERY_TOKEN_LENGTH];
	char	szTime[DISCOVERY_TOKEN_LENGTH];
	
	char	szIpAddrToken[DISCOVERY_TOKEN_ID_LENGTH];
	char	szIpAddr[DISCOVERY_TOKEN_LENGTH];
	
	char	szPortToken[DISCOVERY_TOKEN_ID_LENGTH];
	char	szPort[DISCOVERY_TOKEN_LENGTH];
	
	char	szNameToken[DISCOVERY_TOKEN_ID_LENGTH];
	char	szName[DISCOVERY_TOKEN_LENGTH];

	char	szCamNumber[DISCOVERY_TOKEN_LENGTH];


	ct = clock() + DISCOVERY_TIMEOUT_SECS * CLOCKS_PER_SEC;
	
	tv.tv_sec	= 0;
	tv.tv_usec	= 0;
	size_t		CurLength;
	size_t		NewLength;


	while (clock() <= ct) 
	{
		FD_ZERO( &rset );
		FD_SET( sd, &rset );

		if ( select( sd + 1, &rset, NULL, NULL, &tv ) > 0 ) 
		{
			n = recvfrom(sd, pUdpPacket, DISCOVERY_MAXBUFFER, 0, NULL, NULL);

			// Process the returned packets. 
			pUdpPacket[n] = '\0';	// add a NULL to the close of the string

			sscanf( pUdpPacket, "%s %s %s %s"
								"%s %s"
								"%s %s"
								"%s",
								szResponseToken, szId, szTag, szTime,
								szIpAddrToken, szIpAddr,
								szPortToken, szPort,
								szNameToken );

			strtok(pUdpPacket, "\"");
			strtok(NULL, "\"");
			strtok(NULL, "\"");
			strcpy(szName, strtok(NULL, "\""));

			if ( strcmp("\r\nSessionId=", szName) == 0 )
			{
				strcpy(szName, "Camera Unnamed");
			}

			// Increment number of cameras on network
			(*CameraCount)++;

			sprintf( szCamNumber, "%u ", *CameraCount );

			sprintf( szTemp, "%u 0x%08X %u ", szCamNumber, szIpAddr, szPort );

			// determine string lengths  
			NewLength = strlen( szTemp );
			CurLength = strlen( CameraList );

			CurCamList = (char*)calloc( CurLength + 1, sizeof(char) );

			// copy current string for safe keeping
			strcpy( CurCamList, CameraList );

			// allocate space for combined string
			CameraList = (char*)calloc( CurLength + NewLength + 1, sizeof(char) );

			sprintf( CameraList, "%s%s", CameraList, szTemp );

		}
	}

	CurLength = strlen( CameraList );

	CurCamList = (char*)calloc( CurLength + 1, sizeof(char) );

	// copy current string for safe keeping
	strcpy( CurCamList, CameraList );

	// allocate space for combined string
	CameraList = (char*)calloc( CurLength + sizeof(szCamNumber) + 1, sizeof(char) );

	sprintf( CameraList, "%s%s", szCamNumber, CameraList );

	// Stop the Windows sockets library
	WSACleanup();

	return APNET_SUCCESS;
}


APN_NET_TYPE ApnNetReboot( void )
{
	HINTERNET	hService;
	DWORD		dwBytesAvailable, dwBytesRead;
	char		szUrl[80];
	char*		lpBuffer;

	sprintf(szUrl, "%s%s%s", HTTP_PREAMBLE, g_HostAddr, REBOOT_CMD);

	hService = InternetOpenUrl(g_hSession, szUrl, NULL, 0, 0, 0);
	
	if ( hService == NULL )
	{
		return APNET_ERR_GENERIC_CGI;
	}
	else
	{
		InternetQueryDataAvailable(hService, &dwBytesAvailable, 0, 0);
		lpBuffer = (char*)malloc(dwBytesAvailable+1);
		InternetReadFile(hService, lpBuffer, dwBytesAvailable, &dwBytesRead);
		lpBuffer[dwBytesRead] = NULL;

		// we actually need to check the buffer passed back here.

		free( lpBuffer );
		InternetCloseHandle( hService );
	}

	return APNET_SUCCESS;
}


APN_NET_TYPE ApnNetReadReg( short FpgaReg, unsigned short *FpgaData )
{
	DWORD		dwBytesAvailable, dwBytesRead;
	HINTERNET	hService;
	char		szUrl[80];
	char		*szStopStr;
	char		*lpBuffer;


	// First make sure we have a valid session already
	if ( g_hSession == NULL)
	{
		return APNET_ERR_CONNECT;
	}

	sprintf(szUrl, "%s%s%s%s%d", HTTP_PREAMBLE, g_HostAddr, FPGA_ACCESS, READ_REG, FpgaReg);

	hService = InternetOpenUrl(g_hSession, szUrl, NULL, 0, 0, 0);

	if ( hService == NULL )
	{
		return APNET_ERR_GENERIC_CGI;		// Failure
	}
	else
	{
		InternetQueryDataAvailable(hService, &dwBytesAvailable, 0, 0);
		lpBuffer = (char *)malloc(dwBytesAvailable+1);
		InternetReadFile(hService, lpBuffer, dwBytesAvailable, &dwBytesRead);
		lpBuffer[dwBytesRead] = NULL;

		strtok( lpBuffer, "=");
		
		*FpgaData = (USHORT)strtoul( strtok( NULL, "="), &szStopStr, 16 );

		free(lpBuffer);

		InternetCloseHandle( hService );
	}

	return APNET_SUCCESS;			// Success
}


APN_NET_TYPE ApnNetWriteReg( short FpgaReg, unsigned short FpgaData )
{
	DWORD		dwBytesAvailable, dwBytesRead;
	HINTERNET	hService;
	char		szUrl[80];
	char		*szStopStr;
	char		*lpBuffer;
	USHORT		RetVal;


	// First make sure we have a valid session already
	if ( g_hSession == NULL)
	{
		return APNET_ERR_CONNECT;	// Failure
	}

	sprintf(szUrl, "%s%s%s%s%u&%s0x%X", HTTP_PREAMBLE, g_HostAddr, FPGA_ACCESS, 
		WRITE_REG, FpgaReg, WRITE_DATA, FpgaData);

	hService = InternetOpenUrl(g_hSession, szUrl, NULL, 0, 0, 0);

	if ( hService == NULL )
	{
		return APNET_ERR_GENERIC_CGI;		// Failure
	}
	else
	{
		InternetQueryDataAvailable(hService, &dwBytesAvailable, 0, 0);
		lpBuffer = (char*)malloc(dwBytesAvailable+1);
		InternetReadFile(hService, lpBuffer, dwBytesAvailable, &dwBytesRead);
		lpBuffer[dwBytesRead]=NULL;

		strtok( lpBuffer, "=");
		RetVal = (USHORT)strtoul( strtok( NULL, "="), &szStopStr, 16 );

		/*
		if ( (FpgaData != RetVal) && (FpgaReg != 1) )
		{
			return 1;	// Failed
		}
		*/

		free(lpBuffer);

		InternetCloseHandle( hService );
	}

	return APNET_SUCCESS;	// Success
}


APN_NET_TYPE ApnNetWriteRegMulti( short FpgaReg, unsigned short FpgaData[], unsigned short RegCount )
{
	DWORD		dwBytesAvailable, dwBytesRead;
	HINTERNET	hService;
	char		szUrl[8096];
	char		szTemp[50];
	char		*szStopStr;
	char		*lpBuffer;
	USHORT		RetVal;
	USHORT		LoopCount;
	USHORT		LoopCountRemainder;
	int			i;
	USHORT		StartIndex;
	USHORT		EndIndex;


	// First make sure we have a valid session already
	if ( g_hSession == NULL)
	{
		return APNET_ERR_CONNECT;	// Failure
	}

	LoopCount = (unsigned short)(RegCount / APN_MAX_WRITES_PER_URL);
	LoopCountRemainder = (unsigned short)(RegCount % APN_MAX_WRITES_PER_URL);

	// Process the main blocks of the loop
	for ( i=0; i<LoopCount; i++ )
	{
		// Build the initial URL string
		sprintf(szUrl, "%s%s%s%s%u", HTTP_PREAMBLE, g_HostAddr, FPGA_ACCESS, 
			WRITE_REG, FpgaReg );

		StartIndex = i * APN_MAX_WRITES_PER_URL;
		EndIndex = StartIndex + APN_MAX_WRITES_PER_URL;
		for ( int j=StartIndex; j<EndIndex; j++ )
		{
			sprintf( szTemp, "&%s0x%04X", WRITE_DATA, FpgaData[j] );
			strcat( szUrl, szTemp ); 
		}

		hService = InternetOpenUrl(g_hSession, szUrl, NULL, 0, 0, 0);

		if ( hService == NULL )
		{
			return APNET_ERR_GENERIC_CGI;		// Failure
		}
		else
		{
			InternetQueryDataAvailable(hService, &dwBytesAvailable, 0, 0);
			lpBuffer = (char*)malloc(dwBytesAvailable+1);
			InternetReadFile(hService, lpBuffer, dwBytesAvailable, &dwBytesRead);
			lpBuffer[dwBytesRead]=NULL;

			strtok( lpBuffer, "=");
			RetVal = (USHORT)strtoul( strtok( NULL, "="), &szStopStr, 16 );

			/*
			if ( (FpgaData != RetVal) && (FpgaReg != 1) )
			{
				return 1;	// Failed
			}
			*/

			free(lpBuffer);

			InternetCloseHandle( hService );
		}
	}

	if ( LoopCountRemainder != 0 )
	{
		// Build the initial URL string
		sprintf(szUrl, "%s%s%s%s%u", HTTP_PREAMBLE, g_HostAddr, FPGA_ACCESS, 
			WRITE_REG, FpgaReg );

		StartIndex = LoopCount * APN_MAX_WRITES_PER_URL;
		EndIndex = StartIndex + LoopCountRemainder;
		for ( int j=StartIndex; j<EndIndex; j++ )
		{
			sprintf( szTemp, "&%s0x%04X", WRITE_DATA, FpgaData[j] );
			strcat( szUrl, szTemp ); 
		}

		hService = InternetOpenUrl(g_hSession, szUrl, NULL, 0, 0, 0);

		if ( hService == NULL )
		{
			return APNET_ERR_GENERIC_CGI;		// Failure
		}
		else
		{
			InternetQueryDataAvailable(hService, &dwBytesAvailable, 0, 0);
			lpBuffer = (char*)malloc(dwBytesAvailable+1);
			InternetReadFile(hService, lpBuffer, dwBytesAvailable, &dwBytesRead);
			lpBuffer[dwBytesRead]=NULL;

			strtok( lpBuffer, "=");
			RetVal = (USHORT)strtoul( strtok( NULL, "="), &szStopStr, 16 );

			/*
			if ( (FpgaData != RetVal) && (FpgaReg != 1) )
			{
				return 1;	// Failed
			}
			*/

			free(lpBuffer);

			InternetCloseHandle( hService );
		}
	}

	return APNET_SUCCESS;	// Success
}


APN_NET_TYPE ApnNetStartExp( unsigned short ImageWidth, unsigned short ImageHeight )
{
	HINTERNET	hService;
	DWORD		dwBytesAvailable, dwBytesRead;
	char		szUrl[80];


	// First make sure we have a valid session already
	if ( g_hSession == NULL)
	{
		return APNET_ERR_CONNECT;			// Failure
	}

	// Check to make sure an image isn't already in progress
	if ( g_NetImgInProgress )
	{
		return APNET_ERR_IMAGE_EXPOSING;	// Failure
	}

	// Check to make sure width and height are non-zero
	if ( ( ImageWidth == 0 ) || ( ImageHeight == 0 ) )
	{
		return APNET_ERR_IMAGE_PARAMS;		// Failure
	}

	// Check to make sure our total image size is less than or equal to what we can do
	if ( ( ImageWidth * ImageHeight * 2 ) > ( 1024 * 1024 * 28 ) )
	{
		return APNET_ERR_IMAGE_PARAMS;		// Failure
	}

	///////////////////////////////////////////////////////////////////////
	// time code
	// ci= poll mode is bit 0 
	// ci= zero mem is bit 3
	sprintf(szUrl, "http://%s/FPGA?HBZ=1&CI=0,%u,%u", g_HostAddr, ImageWidth, ImageHeight);
	// sprintf(szUrl, "http://%s/FPGA?CI=0,%u,%u", g_HostAddr, ImageWidth, ImageHeight);

	hService = InternetOpenUrl( g_hSession, 
								szUrl, 
								NULL, 
								0, 
								0, 
								0 );

	if ( hService == NULL )
	{
		return APNET_ERR_GENERIC_CGI;		// Failure
	}
	else
	{
		InternetQueryDataAvailable(hService, &dwBytesAvailable, 0, 0);
		char *lpBuffer = (char*)malloc(dwBytesAvailable+1);
		InternetReadFile(hService, lpBuffer, dwBytesAvailable, &dwBytesRead);
		lpBuffer[dwBytesRead]=NULL;

		free(lpBuffer);

		InternetCloseHandle( hService );
	}

	// Set our global var indicating image size in bytes
	g_NetImgSizeBytes = ImageWidth * ImageHeight * 2;

	g_NetImgInProgress = TRUE;

	return APNET_SUCCESS;	// Success
}


APN_NET_TYPE ApnNetStopExp( bool DigitizeData )
{
	// First make sure we have a valid session already
	if ( g_hSession == NULL)
	{
		return APNET_ERR_CONNECT;			// Failure
	}

	// Check to make sure an image isn't already in progress
	if ( !DigitizeData )
	{
		if ( g_NetImgInProgress )
		{
			g_NetImgInProgress = FALSE;
		}
	}

	return APNET_SUCCESS;	// Success
}


APN_NET_TYPE ApnNetSetSpeed( bool HighSpeed )
{
	// High speed mode is UDP
	// Normal mode is TCP

	if ( HighSpeed )
		g_FastDownload = TRUE;
	else
		g_FastDownload = FALSE;


	return APNET_SUCCESS;	// Success
}


APN_NET_TYPE ApnNetGetImageTcp( unsigned short *pMem )
{
	HINTERNET	hService;
	DWORD		dwBytesAvailable, dwBytesRead;
	DWORD		dwUncompressedSize;
	ULONG		dwImgSize;
	char		szUrl[80];
	char		*pCompressedMem;

	char		szMsg[80];

	// First make sure we have a valid session already
	if ( g_hSession == NULL)
	{
		return APNET_ERR_CONNECT;
	}

	// Check to see if ApnNetStartExp was called first 
	if ( g_NetImgInProgress == FALSE )
	{
		return APNET_ERR_IMAGE_START;		// Failure -- Image never started
	}

	sprintf( szUrl, "http://%s/FILEINFO", g_HostAddr );

	hService = InternetOpenUrl( g_hSession, 
								szUrl, 
								NULL, 
								0,
								INTERNET_FLAG_NO_CACHE_WRITE |
								INTERNET_FLAG_KEEP_CONNECTION,
								0 );

	if ( hService == NULL )
	{
		return APNET_ERR_GENERIC_CGI;		// Failure
	}
	else
	{
		InternetQueryDataAvailable(hService, &dwBytesAvailable, 0, 0);
		char *lpBuffer = (char*)malloc(dwBytesAvailable+1);
		InternetReadFile(hService, lpBuffer, dwBytesAvailable, &dwBytesRead);
		lpBuffer[dwBytesRead]=NULL;

		OutputDebugString( lpBuffer );

		free(lpBuffer);

		InternetCloseHandle( hService );
	}
	
	sprintf( szUrl, "http://%s/UE/image.bin", g_HostAddr );

	hService = InternetOpenUrl( g_hSession, 
								szUrl, 
								NULL, 
								0,
								INTERNET_FLAG_NO_CACHE_WRITE |
								INTERNET_FLAG_KEEP_CONNECTION,
								0 );

	if ( hService == NULL )
	{
		return APNET_ERR_GENERIC_CGI;		// Failure
	}
	else
	{
		/*
		if ( !InternetReadFile( hService, (char *)pMem, g_NetImgSizeBytes, &dwBytesRead ) )
		{
			return APNET_ERR_IMAGE_DATA;	// Failure -- Retrieved incorrect amount of data
		}
		*/
		
		
		InternetQueryDataAvailable( hService, &dwBytesAvailable, 0, 0 );
		pCompressedMem = (char*)malloc( dwBytesAvailable );
		InternetReadFile( hService, (char *)pCompressedMem, dwBytesAvailable, &dwBytesRead );
		

		InternetCloseHandle( hService );
	}
	
	sprintf( szMsg, "dwBytesAvailable:  %u     dwBytesRead:  %u", dwBytesAvailable, dwBytesRead);
	OutputDebugString( szMsg );

	if ( dwBytesRead == g_NetImgSizeBytes )
	{
		memcpy( pMem, pCompressedMem, g_NetImgSizeBytes );
		free( pCompressedMem );
	}
	else
	{
		// check first four chars for "hbz1"
		if ( strncmp( (char *)pMem, "hbz1", 4 ) != 0 )
		{
			OutputDebugString( "Data not compressed" );
			free( pCompressedMem );

			return APNET_ERR_IMAGE_DATA;
		}
		else
		{
			ApnNetDecompress( (unsigned char *)pCompressedMem, 
							  dwBytesRead, 
							  (unsigned char *)pMem, 
							  &dwUncompressedSize );
			
			free( pCompressedMem );
		}
	}

	dwImgSize = g_NetImgSizeBytes / 2;

	for (ULONG j=0; j<dwImgSize; j++)
	{
		pMem[j] = ntohs(pMem[j]);
	}

	g_NetImgInProgress = FALSE;	// Reset to zero so that we'll need another ApnNetStartExp call

	return APNET_SUCCESS;	// Success
}


APN_NET_TYPE GetHostIPAddr( unsigned long *IpAddr )
{
	BOOLEAN		bNameFound;
    sockaddr_in	LocalAddr;
    sockaddr_in	ConnectAddr;
	hostent*	pHostInfo;
	SOCKET		TestSocket;
    // char		szHostName[MAXHOSTNAMELEN];
	char		szHostName[256];
	int			AddrSize; 
	int			RetVal;


	AddrSize = sizeof(sockaddr_in);

	bNameFound = FALSE;

	// Easiest method is to use gethostname() and then gethostbyname()
	// to get the host address.
	/*
    if ( gethostname(szHostName, MAXHOSTNAMELEN) != SOCKET_ERROR ) 
	{
		/* Resolve hostname for local address */
/*		pHostInfo = gethostbyname( szHostName );
		if ( pHostInfo != NULL)
		{
			*IpAddr		= pHostInfo->h_addr;
			bNameFound	= TRUE;
		}
    } 
*/

    // Failed to do it the easy way, use getsockname() since that is the 
	// best way to go.
	if ( !bNameFound ) 
	{
		LocalAddr.sin_addr.s_addr = INADDR_ANY;

		/* Get a UDP socket */
		TestSocket = socket(AF_INET, SOCK_DGRAM, 0);
		
		if (TestSocket != INVALID_SOCKET)
		{
			/* Connect to arbitrary port and address (NOT loopback) */
			ConnectAddr.sin_family		= AF_INET;
			ConnectAddr.sin_port		= htons(IPPORT_ECHO);
			ConnectAddr.sin_addr.s_addr = inet_addr("128.0.0.1");
			
			RetVal = connect( TestSocket,
					   		  (sockaddr *)&ConnectAddr,
							  sizeof(sockaddr_in) );

			if (RetVal != SOCKET_ERROR)
			{
				// no error, let's get the host address
				getsockname( TestSocket, 
							 (sockaddr *)&LocalAddr, 
							 &AddrSize );

				*IpAddr = LocalAddr.sin_addr.s_addr;
				bNameFound = TRUE;
			}

			closesocket( TestSocket );   // close 
		}
   }

	return APNET_SUCCESS;
}




/*

APN_NET_TYPE ApnNetGetImageUdp( unsigned long *MissingBlocks, unsigned short *pMem )
{
	HINTERNET		hService;
	DWORD			dwBytesRead;
	ULONG			IpAddr;
	ULONG			BlockCount;
	PUDPRCV_CONTEXT	pUdpRcv;
	char			UdpBlocks[MAXUDPBLOCKSLEN];
	char			UdpCmd[MAXUDPCMDLEN];
	char			szUrl[MAXUDPCMDLEN];
	char			*pBuf;
	BOOL			bRetVal;


	// First make sure we have a valid session already
	if ( g_hSession == NULL)
	{
		return APNET_ERR_CONNECT;
	}

	// Check to see if ApnNetStartExp was called first 
	if ( g_NetImgInProgress == FALSE )
	{
		return APNET_ERR_IMAGE_START;		// Failure -- Image never started
	}


	start_sockets();

	pBuf = (char *)malloc(MAXBUFSZ);

	BlockCount	= g_NetImgSizeBytes / UDPBLOCKSIZE;
	pUdpRcv		= UdpRcvStart( BlockCount, UDPBLOCKSIZE );

	// GetHostIPAddr( &IpAddr );

	IpAddr = 0x5FEA8C0;
	
	_snprintf( UdpCmd, MAXUDPCMDLEN, HTTP_UDP_CMD, htonl(IpAddr), "B=-1" );
	_snprintf( szUrl, MAXUDPCMDLEN, "%s%s%s", HTTP_PREAMBLE, g_HostAddr, UdpCmd );

	// _snprintf(UdpCmd, MAXUDPCMDLEN, HTTP_UDP_CMD, htonl(gethostid()), "B=-1");

	hService = InternetOpenUrl( g_hSession, 
								szUrl, 
								NULL, 
								0, 0, 0 );
	
	///////////////////////////////////////////////////////////////////////
	// time code
	LARGE_INTEGER	Start, Finish, Total, Frequency;
	double			ElapsedTime;
	long			counter = 0;

	QueryPerformanceFrequency(&Frequency);

	QueryPerformanceCounter( &Start );

	do 
	{
		UdpRcvLoop( pUdpRcv );
	
		if (UdpRcvNext( pUdpRcv, UdpBlocks, MAXUDPBLOCKSLEN ) != NULL ) 
		{
			// printf("UdpBlocks='%s'\n", UdpBlocks);
			counter++;

			bRetVal = InternetReadFile( hService, pBuf, MAXBUFSZ, &dwBytesRead );
			bRetVal = InternetCloseHandle( hService );

			_snprintf( UdpCmd, MAXUDPCMDLEN, HTTP_UDP_CMD, htonl(IpAddr), UdpBlocks );
			_snprintf( szUrl, MAXUDPCMDLEN, "%s%s%s", HTTP_PREAMBLE, g_HostAddr, UdpCmd );

			// _snprintf(UdpCmd, MAXUDPCMDLEN, HTTPUDPCMD, htonl(gethostid()), UdpBlocks);
			// printf("UdpCmd='%s'\n", UdpCmd);

			hService = InternetOpenUrl( g_hSession, 
										szUrl, 
										NULL, 
										0, 0, 0 );
		}
		counter++;
	} while (*UdpBlocks != '\0');

	QueryPerformanceCounter( &Finish );

	Total.QuadPart	= (Finish.QuadPart - Start.QuadPart);
	ElapsedTime		= ((double)Total.QuadPart/(double)Frequency.QuadPart);

	if (counter > 1 )
	{
		ElapsedTime = -ElapsedTime;
	}
	
	// char Message[80];
	// sprintf( Message, "Download Time:  %f", ElapsedTime );
	// MessageBox(NULL, Message, "Info", MB_OK);
	


	// end of time code
	/////////////////////////////////////////////////////////////////////////
	
	UdpRcvNetToHost( pUdpRcv );
	UdpRcvSaveFile( pUdpRcv, "udpimg.bin" );

	memcpy( (char*)pMem, pUdpRcv->ImageBuffer, g_NetImgSizeBytes );
	
	UdpRcvEnd( pUdpRcv );

	bRetVal = InternetReadFile( hService, pBuf, MAXBUFSZ, &dwBytesRead );

	bRetVal = InternetCloseHandle( hService );

	free(pBuf);

	stop_sockets();

	g_NetImgInProgress = FALSE;

	return 0;	// Success
}


*/
// end of udp function

APN_NET_TYPE ApnNetDecompress( unsigned char *Source,
							   unsigned long SourceLength,
							   unsigned char *Dest,
							   unsigned long *DestLength )
{
	unsigned char BytesPerPixel;		// bpp
	unsigned char BytesPerMchb;			// bpm
	unsigned char Mchb;					// mchb

	unsigned long SourceCount	= 0;	// srcx		
	unsigned long DestCount		= 0;	// dstx


	// TBD: Check for "hbz1{bpp}{bpm}{mchb}0{mchb}0" 
	BytesPerPixel	= Source[4];
	BytesPerMchb	= Source[5];
	Mchb			= Source[8];

	SourceCount		= 10;

	while ( SourceCount < SourceLength ) 
	{
		if ( BytesPerMchb == 0 ) 
		{
			// (__BYTE_ORDER == __LITTLE_ENDIAN)
			// Dest[DestCount++] = Source[SourceCount + 1];
			// Dest[DestCount++] = Source[SourceCount + 0];

			Dest[DestCount++] = Source[SourceCount + 0];
			Dest[DestCount++] = Source[SourceCount + 1];

			SourceCount += BytesPerPixel;
		} 
		else if ( Source[SourceCount] == Mchb ) 
		{
			SourceCount++;	// mchb 
			SourceCount++;	// 0x01 

			// (__BYTE_ORDER == __LITTLE_ENDIAN)
			// Dest[DestCount++] = Source[SourceCount + 1];
			// Dest[DestCount++] = Source[SourceCount + 0];

			Dest[DestCount++] = Source[SourceCount + 0];
			Dest[DestCount++] = Source[SourceCount + 1];

			SourceCount += BytesPerPixel;
		} 
		else 
		{
			// (__BYTE_ORDER == __LITTLE_ENDIAN)
			// Dest[DestCount++] = Source[SourceCount++];
			// Dest[DestCount++] = Mchb;

			Dest[DestCount++] = Mchb;
			Dest[DestCount++] = Source[SourceCount++];
		}
	}

	*DestLength = DestCount;
	
	return APNET_SUCCESS;
}

