//////////////////////////////////////////////////////////////////////
//
// ApogeeNetUpload.cpp
//
// Copyright (c) 2003, 2004 Apogee Instruments, Inc.
//
//////////////////////////////////////////////////////////////////////

#include "ApogeeNet.h"
#include "ApogeeNetErr.h"
#include "ApogeeNetUpload.h"


APN_NET_TYPE ApnNetUploadFile( HINTERNET *hSession,
							   char *HostAddr,
							   unsigned char *pBinFile, 
							   unsigned long BinFileLength )
{
	HINTERNET	hHTTPSession;
	HINTERNET	hHTTPReq;
	BOOL		bSend;
	DWORD		dwBytesAvailable, dwBytesRead;
	char		*lpBuffer;
	char		*szMsgBody;
	char		szMsgPrefix[200];
	char		szMsgPostfix[200];
	char		szHeader[200];
	char		szResource[80];
	size_t		MsgBodyLength;
	LPTSTR		szAcceptTypes[2] = { TEXT( "*/*" ), NULL };

	
	// First make sure we have a valid session already
	if ( *hSession == NULL)
	{
		return APN_NET_ERR_CONNECT;
	}

	hHTTPSession = InternetConnect( *hSession,
									HostAddr,
									INTERNET_DEFAULT_HTTP_PORT,
									NULL,
									NULL,
									INTERNET_SERVICE_HTTP,
									0, 0 );
	
	if ( hHTTPSession == NULL )
	{
		return APN_NET_ERR_HTTP_SESSION;  // Failure
	}
	else
	{
		sprintf(szResource, "%s", UPLOAD);

		hHTTPReq = HttpOpenRequest( hHTTPSession,
									TEXT( "POST" ),
									TEXT( szResource ),
									TEXT( "HTTP/1.1" ),
									NULL,
									(LPCTSTR*)szAcceptTypes,
									INTERNET_FLAG_NO_CACHE_WRITE |
									INTERNET_FLAG_KEEP_CONNECTION,
									0 );

		if ( hHTTPReq == NULL )
		{
			InternetCloseHandle( hHTTPSession );

			return APN_NET_ERR_HTTP_OPEN_REQUEST;
		}
		else
		{
			InternetSetOption(	  hHTTPReq, 
							      INTERNET_OPTION_USERNAME, 
							      APOGEE_USERNAME,
							      APOGEE_USERNAME_LENGTH+1 );

			InternetSetOption(    hHTTPReq,
							      INTERNET_OPTION_PASSWORD,
							      APOGEE_PASSWORD,
							      APOGEE_PASSWORD_LENGTH+1 );
				
			sprintf(szMsgPrefix,  "%s--%s%s" 
								  "Content-Disposition: form-data; name=\"File1\"; filename=\"%s\"%s" 
								  "Content-Type: application/octet-stream%s%s",
								  APOGEE_CRLF, APOGEE_BOUNDARY, APOGEE_CRLF,
								  APN_UPLOAD_FILE_NAME, APOGEE_CRLF,
								  APOGEE_CRLF, APOGEE_CRLF);

			sprintf(szMsgPostfix, "%s"
								  "--%s%s" 
								  "Content-Disposition: form-data; name=\"Submit\"%s%s" 
								  "Upload%s" 
								  "--%s--%s",
								  APOGEE_CRLF,
								  APOGEE_BOUNDARY, APOGEE_CRLF,
								  APOGEE_CRLF, APOGEE_CRLF, 
								  APOGEE_CRLF,
								  APOGEE_BOUNDARY, APOGEE_CRLF);

			// Build the message body with the FPGA firmware

			MsgBodyLength	= strlen(szMsgPrefix) + strlen(szMsgPostfix) + BinFileLength;
			szMsgBody		= (char *)malloc(MsgBodyLength);
			ZeroMemory(szMsgBody, MsgBodyLength);
			
			strcpy(szMsgBody, szMsgPrefix);

			ULONG i, index;
 
			index = strlen(szMsgPrefix);

			for (i=0; i<BinFileLength; i++)
			{
				szMsgBody[i+index] = pBinFile[i];
			}

			index += BinFileLength;

			for (i=0; i<strlen(szMsgPostfix); i++)
			{
				szMsgBody[i+index] = szMsgPostfix[i];
			}

			sprintf(szHeader, "Content-Type: multipart/form-data; boundary=%s%s", APOGEE_BOUNDARY, APOGEE_CRLF);

			bSend = HttpAddRequestHeaders( hHTTPReq,
								   szHeader,
								   strlen(szHeader),
								   HTTP_ADDREQ_FLAG_ADD );
					
			if ( !bSend )
			{ 
				MessageBox( NULL, "Failed to add header", "Error", MB_OK );
			}

			bSend = HttpSendRequest( hHTTPReq,
									 NULL, // szHeader,
									 0, // strlen(szHeader),
									 szMsgBody,
									 MsgBodyLength );

			InternetQueryDataAvailable(hHTTPReq, &dwBytesAvailable, 0, 0);
			lpBuffer = (char*)malloc(dwBytesAvailable+1);
			InternetReadFile(hHTTPReq, lpBuffer, dwBytesAvailable, &dwBytesRead);
			free( lpBuffer );

			// clean up
			free( szMsgBody );

			// Check to see if the send request was successful
			if ( !bSend )
			{
				MessageBox( NULL, "APN_NET_ERR_HTTP_SEND_REQUEST", "Error", MB_OK );

				return APN_NET_ERR_HTTP_SEND_REQUEST;			
			}
		}

		InternetCloseHandle( hHTTPReq );
		InternetCloseHandle( hHTTPSession );
	}

	return 0;
}


APN_NET_TYPE ApnNetFlashWrite( HINTERNET *hSession,
							   char *HostAddr,
							   unsigned char *pBinFile,
							   unsigned long BinFileLength )
{
	char		*lpBuffer;
	char		szUrl[80];
	HINTERNET	hService;
	DWORD		dwBytesAvailable, dwBytesRead;
	USHORT		RetVal;


	RetVal = ApnNetUploadFile( hSession, HostAddr, pBinFile, BinFileLength );

	if ( RetVal != APN_NET_SUCCESS )
	{
		return APN_NET_ERR_UPLOAD_FILE;
	}
	else
	{
		// we should really check the manufacturer and device id of the firmware here
		sprintf( szUrl, "%s%s%s%s", HTTP_PREAMBLE, HostAddr, FLASH_ACCESS, FLASH_WRITE );
	
		hService = InternetOpenUrl( *hSession, szUrl, NULL, 0, 0, 0 );

		if ( hService == NULL )
		{
			return APN_NET_ERR_GENERIC_CGI;
		}
		else
		{
			InternetQueryDataAvailable(hService, &dwBytesAvailable, 0, 0);
			lpBuffer = (char*)malloc(dwBytesAvailable+1);
			InternetReadFile(hService, lpBuffer, dwBytesAvailable, &dwBytesRead);
			lpBuffer[dwBytesRead] = NULL;

			// need to check the buffer passed back here.

			free( lpBuffer );
			InternetCloseHandle( hService );
		}
	}

	return 0;
}


APN_NET_TYPE ApnNetLoadDynamicFpga( HINTERNET *hSession,
								    char *HostAddr,
									unsigned char *pFpgaBin, 
									unsigned long FpgaBinLength ) 
{
	char		*lpBuffer;
	char		szUrl[80];
	HINTERNET	hService;
	DWORD		dwBytesAvailable, dwBytesRead;
	USHORT		RetVal;


	RetVal = ApnNetUploadFile( hSession, HostAddr, pFpgaBin, FpgaBinLength );

	if ( RetVal != APN_NET_SUCCESS )
	{
		return APN_NET_ERR_UPLOAD_FILE;
	}
	else
	{
		// we should really check the manufacturer and device id of the firmware here
		sprintf(szUrl, "%s%s%s%s", HTTP_PREAMBLE, HostAddr, FPGA_ACCESS, FPGA_FW_UPLOAD);
	
		hService = InternetOpenUrl( *hSession, szUrl, NULL, 0, 0, 0 );

		if ( hService == NULL )
		{
			return APN_NET_ERR_GENERIC_CGI;
		}
		else
		{
			InternetQueryDataAvailable(hService, &dwBytesAvailable, 0, 0);
			lpBuffer = (char*)malloc(dwBytesAvailable+1);
			InternetReadFile(hService, lpBuffer, dwBytesAvailable, &dwBytesRead);
			lpBuffer[dwBytesRead] = NULL;

			// need to check the buffer passed back here.

			free( lpBuffer );
			InternetCloseHandle( hService );
		}
	}

	return 0;
}


APN_NET_TYPE ApnNetClearFpga( HINTERNET *hSession, char *HostAddr )
{
	HINTERNET	hService;
	DWORD		dwBytesAvailable, dwBytesRead;
	char		szUrl[80];
	char*		lpBuffer;

	sprintf(szUrl, "%s%s%s%s", HTTP_PREAMBLE, HostAddr, FPGA_ACCESS, FPGA_FW_CLEAR);

	hService = InternetOpenUrl( *hSession, szUrl, NULL, 0, 0, 0 );
	
	if ( hService == NULL )
	{
		return APN_NET_ERR_GENERIC_CGI;
	} 
	else
	{
		InternetQueryDataAvailable(hService, &dwBytesAvailable, 0, 0);
		lpBuffer = (char*)malloc(dwBytesAvailable+1);
		InternetReadFile(hService, lpBuffer, dwBytesAvailable, &dwBytesRead);
		lpBuffer[dwBytesRead] = NULL;

		// we actually need to check the buffer passed back here.
		// make sure that "done=0" is what we get back

		free( lpBuffer );
		InternetCloseHandle( hService );
	}

	return APN_NET_SUCCESS;
}


APN_NET_TYPE ApnNetSetMacAddress( HINTERNET *hSession,
								  char *HostAddr, 
								  char *pMacAddr )
{
	HINTERNET	hService;
	DWORD		dwBytesAvailable, dwBytesRead;
	USHORT		RetVal;
	char		*lpBuffer;
	char		szUrl[80];


	RetVal = ApnNetUploadFile( hSession, HostAddr, (unsigned char *)pMacAddr, NVRAM_LEN_MAC_ADDR );

	if ( RetVal != APN_NET_SUCCESS )
	{
		return APN_NET_ERR_UPLOAD_FILE;
	}
	else
	{
		sprintf(szUrl, "%s%s%s%s", HTTP_PREAMBLE, HostAddr, NVRAM_ACCESS, NVRAM_SET_MAC_ADDR );
	
		hService = InternetOpenUrl( *hSession, szUrl, NULL, 0, 0, 0 );

		if ( hService == NULL )
		{
			return APN_NET_ERR_GENERIC_CGI;
		}
		else
		{
			InternetQueryDataAvailable(hService, &dwBytesAvailable, 0, 0);
			lpBuffer = (char*)malloc(dwBytesAvailable+1);
			InternetReadFile(hService, lpBuffer, dwBytesAvailable, &dwBytesRead);
			lpBuffer[dwBytesRead] = NULL;

			// need to check the buffer passed back here.

			free( lpBuffer );
			InternetCloseHandle( hService );
		}
	}

	return APN_NET_SUCCESS;
}


APN_NET_TYPE ApnNetSetCameraName( HINTERNET *hSession,
								  char *HostAddr,
								  char *pCameraName )
{
	HINTERNET	hService;
	DWORD		dwBytesAvailable, dwBytesRead;
	USHORT		RetVal;
	char		*lpBuffer;
	char		szUrl[80];
	char		szName[NVRAM_LEN_CAMERA_NAME];


	if ( strlen(pCameraName) < NVRAM_LEN_CAMERA_NAME )
	{
		RetVal = ApnNetUploadFile( hSession, HostAddr, (unsigned char *)pCameraName, NVRAM_LEN_CAMERA_NAME );
	}
	else
	{
		// The camera name string that was passed in was too big, we need
		// to make sure it is an appropriate number of bytes for NVRAM storage.
		memcpy( szName, pCameraName, NVRAM_LEN_CAMERA_NAME-1 );
		szName[NVRAM_LEN_CAMERA_NAME-1] = '\0';
		RetVal = ApnNetUploadFile( hSession, HostAddr, (unsigned char *)szName, NVRAM_LEN_CAMERA_NAME );
	}

	if ( RetVal != APN_NET_SUCCESS )
	{
		return APN_NET_ERR_UPLOAD_FILE;
	}
	else
	{
		sprintf(szUrl, "%s%s%s%s", HTTP_PREAMBLE, HostAddr, NVRAM_ACCESS, NVRAM_SET_CAMERA_NAME );
	
		hService = InternetOpenUrl( *hSession, szUrl, NULL, 0, 0, 0 );

		if ( hService == NULL )
		{
			return APN_NET_ERR_GENERIC_CGI;
		}
		else
		{
			InternetQueryDataAvailable(hService, &dwBytesAvailable, 0, 0);
			lpBuffer = (char*)malloc(dwBytesAvailable+1);
			InternetReadFile(hService, lpBuffer, dwBytesAvailable, &dwBytesRead);
			lpBuffer[dwBytesRead] = NULL;

			// need to check the buffer passed back here.

			free( lpBuffer );
			InternetCloseHandle( hService );
		}
	}

	return APN_NET_SUCCESS;
}


APN_NET_TYPE ApnNetGetCameraName( HINTERNET *hSession,
								  char *HostAddr,
								  char *pCameraName )
{
	HINTERNET	hService;
	APN_NET_TYPE	ApStatus;
	DWORD		dwBytesAvailable, dwBytesRead;
	char		*lpBuffer;
	char		szUrl[80];

	
	// First make sure we have a valid session already
	if ( *hSession == NULL)
	{
		return APN_NET_ERR_CONNECT;
	}

	sprintf(szUrl, "%s%s%s%s", HTTP_PREAMBLE, HostAddr, NVRAM_ACCESS, NVRAM_GET_CAMERA_NAME );
	
	hService = InternetOpenUrl( *hSession, szUrl, NULL, 0, 0, 0 );

	if ( hService == NULL )
	{
		return APN_NET_ERR_GENERIC_CGI;
	}
	else
	{
		InternetQueryDataAvailable(hService, &dwBytesAvailable, 0, 0);
		lpBuffer = (char*)malloc(dwBytesAvailable+1);
		InternetReadFile(hService, lpBuffer, dwBytesAvailable, &dwBytesRead);
		lpBuffer[dwBytesRead] = NULL;

		// need to check the buffer passed back here.

		free( lpBuffer );
		InternetCloseHandle( hService );
	}

	ApStatus = ApnNetGetNvramData( hSession, HostAddr, pCameraName );

	return APN_NET_SUCCESS;
}


