#include "udprcv.h"

#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#include <winsock2.h>

// #include "winsockx.h"
// #include "s_io.h"
// #include "network.h"


/* ----------------------------------------------------------------------------
Defines
---------------------------------------------------------------------------- */

// #if (defined(_WIN32))
#define snprintf _snprintf
// #endif /* #if (defined(_WIN32)) */


/* ----------------------------------------------------------------------------
Global Variables
---------------------------------------------------------------------------- */


/* ----------------------------------------------------------------------------
Function Prototypes
---------------------------------------------------------------------------- */

int s_readable_timeo(int fd, int sec, int usec);


/* ----------------------------------------------------------------------------
Functions
---------------------------------------------------------------------------- */

// #if defined(_WIN32)

void start_sockets(void) {
  WSADATA stWSAData;
  int nRet;

  nRet = WSAStartup(MAKEWORD(1, 0), &stWSAData);
} /* start_sockets */

void stop_sockets(void) {
  WSACleanup();
} /* stop_sockets */

void sleep(int seconds) {
  Sleep(seconds * 1000); /* Sleep takes milliseconds */
} /* sleep */

/*
#else

void start_sockets(void) {
} // start_sockets 

void stop_sockets(void) {
} // stop_sockets 

#endif

*/


#if (defined(_WIN32))

/* ----------------------------------------------------------------------------
gethostid
---------------------------------------------------------------------------- */
/*
long int gethostid(void) {
  return (GetHostID());
} /* gethostid */


/* ----------------------------------------------------------------------------
inet_aton
---------------------------------------------------------------------------- */
int inet_aton(const char *strptr, struct in_addr *addrptr) {
  unsigned long a;
  
  
  a = inet_addr(strptr);
  if (a == INADDR_NONE) return (0);
  addrptr->s_addr = a;
  return (1);
} /* inet_aton */

#endif /* #if (defined(_WIN32)) */


/* ----------------------------------------------------------------------------
inet_pton
---------------------------------------------------------------------------- */
int inet_pton(int family, const char *strptr, void *addrptr) {
  if (family == AF_INET) {
    struct in_addr in_val;

    if (inet_aton(strptr, &in_val)) {
      memcpy(addrptr, &in_val, sizeof(struct in_addr));
      return (1);
    }
    return (0);
  }
#if defined(_WIN32)
  /* TBD: _WIN32 support */
#else
  errno = EAFNOSUPPORT;
#endif
  return (-1);
} /* inet_pton */


/* ----------------------------------------------------------------------------
UdpReceiveStart
---------------------------------------------------------------------------- */
PUDPRCV_CONTEXT UdpRcvStart(long BlockCount, long BlockSize)
{
	PUDPRCV_CONTEXT pUdpRcv;
	struct sockaddr_in servaddr;

	pUdpRcv = (PUDPRCV_CONTEXT)malloc(sizeof(UDPRCV_CONTEXT));
	
	if ( pUdpRcv != NULL ) 
	{
		memset(pUdpRcv, 0, sizeof(UDPRCV_CONTEXT));

		pUdpRcv->sd = socket(AF_INET, SOCK_DGRAM, 0);

		memset(&servaddr, 0, sizeof(servaddr));
		servaddr.sin_family			= AF_INET;
		servaddr.sin_addr.s_addr	= htonl(INADDR_ANY);
		servaddr.sin_port			= htons(CECSVC_PORT);
		bind(pUdpRcv->sd, (struct sockaddr *)&servaddr, sizeof(servaddr));

		pUdpRcv->BlockCount			= BlockCount;
		pUdpRcv->BlockSize			= BlockSize;
		pUdpRcv->BlockReceived		= (char *)malloc(BlockCount * sizeof(char));

		if (pUdpRcv->BlockReceived == NULL)
		{
			closesocket(pUdpRcv->sd);
			free(pUdpRcv);
			return (NULL);
		}
    
		memset(pUdpRcv->BlockReceived, 0, BlockCount * sizeof(char));

		// pUdpRcv->ImageBuffer = (char *)malloc(BlockCount * (2 * sizeof(long) + BlockSize));
		// pUdpRcv->ImageBuffer = (char *)malloc(BlockCount * BlockSize);
		
		// this is for the "region" method
		// pUdpRcv->ImageBuffer = (char *)malloc(200 * BlockSize);

		pUdpRcv->ImageBuffer = (char *)malloc( BlockCount * BlockSize );


		if (pUdpRcv->ImageBuffer == NULL) 
		{
			closesocket(pUdpRcv->sd);
			free(pUdpRcv->BlockReceived);
			free(pUdpRcv);
			return (NULL);
		}

		// memset(pUdpRcv->ImageBuffer, 0, BlockCount * (2 * sizeof(long) + BlockSize));
		// memset(pUdpRcv->ImageBuffer, 0, (BlockCount * BlockSize));
		
		// This is for the "region" method
		// memset(pUdpRcv->ImageBuffer, 0, (200 * BlockSize));

		memset(pUdpRcv->ImageBuffer, 0, (BlockCount * BlockSize));
	}

	return ( pUdpRcv );

} /* UdpRcvStart */


/* ----------------------------------------------------------------------------
UdpRcvProcessBuffer

Returns
  0: success
  otherwise: error
---------------------------------------------------------------------------- */
int UdpRcvProcessBuffer( PUDPRCV_CONTEXT pUdpRcv, PUDPRCV_BUFFER pUdpRcvBuf ) 
{
	long BlockNdx;
	long BlockSz;

	/*
	BlockNdx	= pUdpRcvBuf->BlockIndex;
	BlockNdx	= ntohl(BlockNdx);
	BlockSz		= pUdpRcvBuf->BlockSize;
	BlockSz		= ntohl(BlockSz);
	*/

	BlockNdx	= ntohl(pUdpRcvBuf->BlockIndex);
	BlockSz		= ntohl(pUdpRcvBuf->BlockSize);

	// printf("%ld, 0x%lX\n", BlockNdx, BlockSz); 

	if (BlockNdx >= pUdpRcv->BlockCount) 
	{
		return (1);
	}

	if ( pUdpRcv->BlockSize == 0 ) 
	{
		pUdpRcv->BlockSize = BlockSz;
	} 
	else if ( pUdpRcv->BlockSize != BlockSz ) 
	{
		return (2);
	}

	pUdpRcv->BlockReceived[BlockNdx] = 1;

	memcpy( pUdpRcv->ImageBuffer + BlockNdx * BlockSz,
			pUdpRcvBuf->BlockBuffer, 
			BlockSz );

	return (0);

} /* UdpRcvProcessBuffer */


/* ----------------------------------------------------------------------------
UdpRcvLoop
---------------------------------------------------------------------------- */
void UdpRcvLoop( PUDPRCV_CONTEXT pUdpRcv ) 
{
	// UDPRCV_BUFFER	UdpRcvBuf;
	char			UdpRcvBuf[UDPBLOCKSIZE];
	clock_t			ct;
	fd_set			rset;
	struct timeval	tv;
	unsigned int	sd;
	int				n;


#if (USE_TIMEOUT_TYPE == 1)
	
	/* Get UDP packets until TIMEOUT_SECS elapsed from last UDP packet. */
	ct = clock() + (long)(TIMEOUT_SECS * CLOCKS_PER_SEC);
	
	tv.tv_sec	= 0;
	tv.tv_usec	= 0;
	sd			= pUdpRcv->sd;
	
	unsigned int	newSd;
	long			timeVal;
	clock_t			newclock;
	newSd			= sd + 1;
	timeVal			= (long)(TIMEOUT_SECS * CLOCKS_PER_SEC);

	long BlockNdx;

	BlockNdx = 0;

	pUdpRcv->BlockCount = 0;

	do 
	{
		// tv.tv_sec	= 0;
		// tv.tv_usec	= 0;
		
		FD_ZERO(&rset);
		FD_SET(sd, &rset);

		// if (s_readable_timeo(pUdpRcv->sd, 0, 0) > 0) 
		if ( select( newSd/* sd + 1 */, &rset, NULL, NULL, &tv) > 0 )
		{
			// n = recvfrom(pUdpRcv->sd, (char *)&UdpRcvBuf, MAXUDPPACKETSIZE, 0, NULL, NULL);
			// recvfrom(pUdpRcv->sd, UdpRcvBuf, UDPBLOCKSIZE, 0, NULL, NULL);
			recvfrom(pUdpRcv->sd, pUdpRcv->ImageBuffer + BlockNdx * UDPBLOCKSIZE, UDPBLOCKSIZE, 0, NULL, NULL);
			pUdpRcv->BlockReceived[BlockNdx] = 1;
			/*
			memcpy( pUdpRcv->ImageBuffer + BlockNdx * UDPBLOCKSIZE,
					UdpRcvBuf,
					UDPBLOCKSIZE );
			*/
			BlockNdx++;

			pUdpRcv->BlockCount = BlockNdx;

			// recvfrom(pUdpRcv->sd, (char *)&UdpRcvBuf, MAXUDPPACKETSIZE, 0, NULL, NULL);
			// UdpRcvProcessBuffer(pUdpRcv, &UdpRcvBuf);
			ct = clock() + timeVal; /* (long)(TIMEOUT_SECS * CLOCKS_PER_SEC); /* clock() + CLOCKS_PER_SEC / 3; */

			// newclock = clock();
			// ct = newclock + timeVal;
		}
	} while ( clock() <= ct);

#elif (USE_TIMEOUT_TYPE == 2)

	/* Wait TIMEOUT_SECS for UDP replies. */
	ct = clock() + TIMEOUT_SECS * CLOCKS_PER_SEC;

	while (clock() <= ct) 
	{
		tv.tv_sec	= 0;
		tv.tv_usec	= 0;

		sd = pUdpRcv->sd;
		FD_ZERO(&rset);
		FD_SET(sd, &rset);

		// if (s_readable_timeo(pUdpRcv->sd, 0, 0) > 0) 
		if ( select(sd + 1, &rset, NULL, NULL, &tv) > 0 )
		{
			n = recvfrom(pUdpRcv->sd, &UdpRcvBuf, MAXUDPPACKETSIZE, 0, NULL, NULL);
			UdpRcvProcessBuffer(pUdpRcv, &UdpRcvBuf);
		}
	}

#endif

	return;

} /* UdpRcvLoop */


/* ----------------------------------------------------------------------------
UdpRcvNext
---------------------------------------------------------------------------- */
char *UdpRcvNext( PUDPRCV_CONTEXT pUdpRcv, char *pBuf, int MaxBufLen ) 
{
	char	TmpBuf[MAXTMPLEN];
	int		TmpLen;
	int		BufLen;
	int		i;


	if (pBuf == NULL) 
	{
		return (NULL);
	}
	*pBuf = '\0';
	BufLen = 0;

	for (i = 0; i < pUdpRcv->BlockCount; i++) 
	{
		if (pUdpRcv->BlockReceived[i] == 0) 
		{
			TmpLen = snprintf(TmpBuf, MAXTMPLEN, "B=%d&", i);
			if (BufLen + TmpLen < MaxBufLen) 
			{
				strcpy(pBuf + BufLen, TmpBuf);
				BufLen += TmpLen;
			}
		}
	}

	if ( pBuf[BufLen - 1] == '&' )
	{
		pBuf[BufLen - 1] = '\0';
	}

	return ((BufLen == 0)? NULL: pBuf);

} /* UdpRcvNext */


/* ----------------------------------------------------------------------------
UdpRcvNetToHost
  Convert the image pixels from network byte order to host byte order.
---------------------------------------------------------------------------- */
void UdpRcvNetToHost( PUDPRCV_CONTEXT pUdpRcv ) 
{
#if (BYTESPERPIXEL == 1)

#elif (BYTESPERPIXEL == 2)
	long MaxShorts;
	long i;
	short *p;

	MaxShorts = pUdpRcv->BlockSize / sizeof(short) * pUdpRcv->BlockCount;
	p = (short *)pUdpRcv->ImageBuffer;
	for (i = 0; i < MaxShorts; i++) 
	{
		*p = ntohs(*p);
		p++;
	}

#elif (BYTESPERPIXEL == 4)
	long MaxLongs;
	long i;
	long *p;

	MaxLongs = pUdpRcv->BlockSize / sizeof(long) * pUdpRcv->BlockCount;
	p = (long *)pUdpRcv->ImageBuffer;
	for (i = 0; i < MaxLongs; i++) 
	{
		*p = ntohl(*p);
		p++;
	}

#endif

} /* UdpRcvNetToHost */


/* ----------------------------------------------------------------------------
UdpRcvSaveFile
---------------------------------------------------------------------------- */
int UdpRcvSaveFile( PUDPRCV_CONTEXT pUdpRcv, char *ofn )
{
	FILE *ofp;

	ofp = fopen(ofn, "wb");
	if (ofp == NULL) 
	{
		return (1);
	}

	fwrite( pUdpRcv->ImageBuffer, 
			sizeof(char), 
			pUdpRcv->BlockCount * pUdpRcv->BlockSize, 
			ofp );

	fclose(ofp);
  
	return (0);

} /* UdpRcvSaveFile */


/* ----------------------------------------------------------------------------
UdpRcvEnd
---------------------------------------------------------------------------- */
void UdpRcvEnd( PUDPRCV_CONTEXT pUdpRcv ) 
{
	if (pUdpRcv == NULL)
	{
		return;
	}

	if (pUdpRcv->ImageBuffer != NULL) 
	{
		free(pUdpRcv->ImageBuffer);
		pUdpRcv->ImageBuffer = NULL;
	}

	closesocket(pUdpRcv->sd);
	free(pUdpRcv);
	pUdpRcv = NULL;

} /* UdpRcvEnd */


/* ----------------------------------------------------------------------------
UdpTrimAndSave
---------------------------------------------------------------------------- */
void UdpTrimAndSave ( PUDPRCV_CONTEXT pUdpRcv, char *pImgData )
{
	long BlockCount;
	long BlockSize;
	long Index;
	long *BlockNumber;
	long i;


	if (pUdpRcv == NULL)
	{
		return;
	}

	BlockCount	= pUdpRcv->BlockCount - 1;
	BlockSize	= pUdpRcv->BlockSize;

	for (i=0; i<BlockCount; i++)
	{
		// BlockNumber = (long)(pUdpRcv->ImageBuffer + (i * BlockSize) );
		BlockNumber = (long *)pUdpRcv->ImageBuffer;
		BlockNumber += i * (BlockSize / 4);

		/*
		memcpy( pImgData + (*BlockNumber * BlockSize),
				pUdpRcv->ImageBuffer + (i * BlockSize) + 8,
				BlockSize );
		*/
		memcpy( pImgData + (*BlockNumber * BlockSize),
				pUdpRcv->ImageBuffer + (i * BlockSize) + 8,
				BlockSize );
	}

} /* UdpTrimAndSave */

